package mn.grapecity.mmcomponent.mm_component;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.provider.Settings;
import android.util.Log;

import androidx.annotation.NonNull;

import org.json.JSONException;
import org.json.JSONObject;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.content.Context.MODE_PRIVATE;

/**
 * MmComponentPlugin
 */
public class MmComponentPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private static Result methodResult;
    private static Activity activity;
    private MethodChannel channel;

    @Override
    public void onAttachedToActivity(ActivityPluginBinding activityPluginBinding) {
        activity = activityPluginBinding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding activityPluginBinding) {
    }

    @Override
    public void onDetachedFromActivity() {
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "mm_component");
        channel.setMethodCallHandler(this);
    }

    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "mm_component");
        channel.setMethodCallHandler(new MmComponentPlugin());
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("open")) {
            methodResult = result;
            String operation = call.argument("operation");
            String version = call.argument("version");
            String url = call.argument("url");
            boolean openNewComponent = call.argument("openNewComponent");
            String failAlertMode = call.argument("failAlertMode");

            if (openNewComponent) {
                SharedPreferences settings = activity.getSharedPreferences("NativeStorage", MODE_PRIVATE);
                String biometricStatus = settings.getString(MMLoginHelper.KEY_BIOMETRIC_STATUS, "false");

                Intent intent = new Intent(activity, MMLoginActivity.class);
                intent.putExtra(MMLoginHelper.KEY_OPERATION, operation);
                intent.putExtra(MMLoginHelper.KEY_APP_VERSION, version);
                intent.putExtra(MMLoginHelper.KEY_BIOMETRIC_STATUS, biometricStatus);
                intent.putExtra(MMLoginHelper.KEY_FAIL_MODE, failAlertMode);
                intent.setType(Settings.ACTION_SYNC_SETTINGS);
                intent.putExtra(MMLoginHelper.KEY_BASE_URL, url);
                activity.startActivity(intent);
            }
        } else {
            result.notImplemented();
        }
    }

    public static void setBiometric(Boolean biometric) {
        SharedPreferences settings = activity.getSharedPreferences("NativeStorage", MODE_PRIVATE);
        SharedPreferences.Editor editor = settings.edit();
        editor.putString(MMLoginHelper.KEY_BIOMETRIC_STATUS, biometric.toString());
        editor.apply();
    }

    public static void dispatchResult(int status, JSONObject data) {
        Log.d("dispatchResult:", " " + data + " status: " + status);

        if (status == MMLoginHelper.RESULT_CODE_SUCCESS) {
            try {
                if (data.has("biometric")) {
                    MmComponentPlugin.setBiometric(data.getBoolean("biometric"));
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }

            MmComponentPlugin.hasResponse("onSuccess", data.toString());

        } else if (status == MMLoginHelper.RESULT_CODE_FAILED) {
            try {
                if (data.has("biometric")) {
                    MmComponentPlugin.setBiometric(data.getBoolean("biometric"));
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }

            MmComponentPlugin.hasResponse("onFail", data.toString());

        } else if (status == MMLoginHelper.RESULT_CODE_GOTO) {
            MmComponentPlugin.hasResponse("onGoto", data.toString());
        }
    }

    public static void hasResponse(String resultCode, String resultData) {
        try {
            JSONObject obj = new JSONObject();
            obj.put("resultCode", resultCode);
            obj.put("resultData", resultData);
            methodResult.success(obj.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
