//
//  KeyChainService.swift
//  LoginComp
//
//  Created by Grapecity on 8/10/20.
//  Copyright © 2020 Grapecity. All rights reserved.
//

import Foundation
import Security

// Constant Identifiers
let userAccount = "AuthenticatedUser"
let accessGroup = "SecuritySerivice"


/**
 *  User defined keys for new entry
 *  Note: add new keys for new secure item and use them in load and save methods
 */

let _phoneKey: NSString = "#$nalagma-m-c-g-enohp#"
let _fpEnabled: NSString = "#$nalagma-m-c-g-fp-enabled#"
let _UDID: NSString = "#$mostmoney-m-c-g-udid#"

// Arguments for the keychain queries
let kSecClassValue = NSString(format: kSecClass)
let kSecAttrAccountValue = NSString(format: kSecAttrAccount)
let kSecValueDataValue = NSString(format: kSecValueData)
let kSecClassGenericPasswordValue = NSString(format: kSecClassGenericPassword)
let kSecAttrServiceValue = NSString(format: kSecAttrService)
let kSecMatchLimitValue = NSString(format: kSecMatchLimit)
let kSecReturnDataValue = NSString(format: kSecReturnData)
let kSecMatchLimitOneValue = NSString(format: kSecMatchLimitOne)

let _mobileKey = "BIDLOGINMOBILE"
let _fbRemember = "BIDFBREMEMBER"
let _mobileRemember = "BIDREMEMBERMOBILE"

internal class KeychainService: NSObject {
    
    /**
     * Exposed methods to perform save and load queries.
     */
    
    public class func savePhone(token: NSString) {
        self.save(service: _phoneKey, data: token)
    }
    
    public class func clearPhone() {
        self.save(service: _phoneKey, data: "")
    }
    
    public class func loadPhone() -> NSString? {
        if let mobile: NSString = persist.object(forKey: _mobileKey) as? NSString {
            self.save(service: _phoneKey, data: mobile)
            persist.set(nil, forKey: _mobileKey)
            persist.removeObject(forKey: _mobileKey)
        }
        return self.load(service: _phoneKey)
    }
    
    public class func biometryEnabled() -> Bool {
        if let biometry: NSString = persist.object(forKey: _fbRemember) as? NSString {
            self.save(service: _fpEnabled, data: biometry)
            persist.set(nil, forKey: _fbRemember)
            persist.removeObject(forKey: _fbRemember)
        }
        
        if let value = self.load(service: _fpEnabled){
            return  value == "1" ? true : false
        }
        else{
            return false
        }
    }
    
    public class func biometryDisable() {
        self.biometryEnable(enable: false)
    }
    
    public class func biometryEnable(enable: Bool) {
        self.save(service: _fpEnabled, data: enable ? "1" : "0")
    }
    
    public class func saveBiometryCounter(_ mobileNumber: String, count: Int) {
        let data = String(count) as NSString
        self.save(service: mobileNumber as NSString, data: data)
    }
    
    public class func biometryCounter(_ mobileNumber: String) -> Int {
        guard let data = self.load(service: mobileNumber as NSString) else {
            return 0
        }
        
        if data == "" {
            return 0
        }
        
        return Int(data.intValue)
    }
    
    public class func biometryCounterReset(_ mobilNumber: String) {
        self.save(service: mobilNumber as NSString, data: "")
    }
    
    public class func getUDID() -> String? {
        guard let data = self.load(service: _UDID) else {
            return nil
        }
        
        return String(data)
    }
    
    public class func saveUDID(_ udid: String) {
        self.save(service: _UDID, data: udid as NSString)
    }
    
    /**
     * Internal methods for querying the keychain.
     */
    
    private class func save(service: NSString, data: NSString) {
        let dataFromString: NSData = data.data(using: String.Encoding.utf8.rawValue, allowLossyConversion: false)! as NSData
        
        // Instantiate a new default keychain query
        let keychainQuery: NSMutableDictionary = NSMutableDictionary(objects: [kSecClassGenericPasswordValue, service, userAccount, dataFromString], forKeys: [kSecClassValue, kSecAttrServiceValue, kSecAttrAccountValue, kSecValueDataValue])
        
        // Delete any existing items
        SecItemDelete(keychainQuery as CFDictionary)
        
        // Add the new keychain item
        SecItemAdd(keychainQuery as CFDictionary, nil)
    }
    
    private class func load(service: NSString) -> NSString? {
        // Instantiate a new default keychain query
        // Tell the query to return a result
        // Limit our results to one item
        let keychainQuery: NSMutableDictionary = NSMutableDictionary(objects: [kSecClassGenericPasswordValue, service, userAccount, kCFBooleanTrue ?? true, kSecMatchLimitOneValue], forKeys: [kSecClassValue, kSecAttrServiceValue, kSecAttrAccountValue, kSecReturnDataValue, kSecMatchLimitValue])
        
        var dataTypeRef :AnyObject?
        
        // Search for the keychain items
        let status: OSStatus = SecItemCopyMatching(keychainQuery, &dataTypeRef)
        var contentsOfKeychain: NSString? = nil
        
        if status == errSecSuccess {
            if let retrievedData = dataTypeRef as? NSData {
                contentsOfKeychain = NSString(data: retrievedData as Data, encoding: String.Encoding.utf8.rawValue)
                guard let content = contentsOfKeychain else { return nil }
                if content.length == 0 { return nil }
            }
        } else {
            return nil
        }
        
        return contentsOfKeychain
    }
}

