//
//  BaseWK.swift
//  LoginComp
//
//  Created by Grapecity on 7/2/20.
//  Copyright © 2020 Grapecity. All rights reserved.
//

import Foundation
import WebKit
import LocalAuthentication
import os

public class LoginVC: UIViewController, WKUIDelegate, WKNavigationDelegate {
    
    
    // MARK:- Variables
    
    public var delegate: LoginCompDelegate?
    
    public var externalURL: String?
    public var biometricStatus: Bool = false
    public var appVersion: String?
    public var failAlertMode: String?
    
    private var fullURL = ""
    private var navigation: LCNavigation!
    
    private let deviceId = LoginHeader.DEVICE_ID + "=" + getUDID()
    private let deviceName = LoginHeader.DEVICE_NAME + "=" + getDeviceName()
    private let deviceIp = LoginHeader.DEVICE_IP + "=" + getWiFiAddress()
    private let deviceMac = LoginHeader.DEVICE_MAC + "=" + getUDID()
    private let osType = LoginHeader.OS_TYPE + "=" + getOsType()
    private var version: String {
        get {
            guard let vers = appVersion else {
                return LoginHeader.APP_VERSION + "=" + getAppVersion()
            }
            return LoginHeader.APP_VERSION + "=" + vers
        }
    }
    private let osVersion = LoginHeader.OS_VERSION + "=" + getOsVersion()
    
    private var vwMain: UIView = {
        let vw = UIView()
        vw.backgroundColor = UIColor.clear
        return vw
    }()
    
    private var webView: WKWebView = {
        let web = WKWebView()
        web.backgroundColor = UIColor.clear
        return web
    }()
    
    //MARK:- Init
    public convenience init(navigation: LCNavigation) {
        self.init()
        self.navigation = navigation
    }
    
    // MARK:- Functions
    private func loadURL() {
        
        if var url = externalURL {
            
            while url.last == "/" {
                url = String(url.removeLast())
            }
            
            switch navigation {
            case .KEY_NAVIGATION_LOGIN:
                fullURL = url + getSubURL(navigation) + "?" + deviceId + "&" + deviceName + "&" + deviceIp + "&" + deviceMac + "&" + osType + "&" + version + "&" + osVersion
                if let mobile = KeychainService.loadPhone() as String?, persist.bool(forKey: _mobileRemember) == true {
                    fullURL = fullURL + ("&" + LoginHeader.MOBILE + "=" + mobile) + ("&" + LoginHeader.REMEMBER_ME + "=true")
                }
                var count = persist.integer(forKey: "biometryCount")
                if isBiometryAvailable() && !biometricStatus {
                    if count < 4 {
                        switch BiometricType {
                        case .faceID:
                            fullURL = fullURL + "&" + (LoginHeader.BIOMETRIC_TYPE + "=FID")
                        case .touchID:
                            fullURL = fullURL + "&" + (LoginHeader.BIOMETRIC_TYPE + "=TID")
                        default:
                            break
                        }
                        count += 1
                        persist.set(count, forKey: "biometryCount")
                    }
                } else if(isBiometryAvailable()) {
                    if count >= 4 {
                        fullURL = fullURL + "&" + (LoginHeader.BIOMETRIC_TYPE + "=enabled")
                    }
                }
            case .KEY_NAVIGATION_CHANGEPIN:
                fallthrough
            case .KEY_NAVIGATION_CHANGEMOBILE:
                fallthrough
            case .KEY_NAVIGATION_PIN:
                fullURL = url + getSubURL(navigation)
            default:
                didFail(desc: "wrong navigation key", code: 101)
                return
            }
            
            loadURL(url: fullURL)
        }
        else {
            didFail(desc: "externalurl is not set", code: 101)
        }
    }
    
    private func loadURL(url: String) {
        let fullURL = url.replacingOccurrences(of: " ", with: "%20")
        
        if let tmpURL = URL(string: fullURL) {
            let req = URLRequest(url: tmpURL)
            
            self.webView.configuration.preferences.javaScriptEnabled = true
            self.webView.configuration.userContentController.addUserScript(self.getZoomDisableScript())
            self.webView.load(req)
            self.webView.uiDelegate = self
            self.webView.navigationDelegate = self
        }
        else {
            didFail(desc: "wrong url", code: 101)
        }
    }
    
    private func parseValues(_ items: [URLQueryItem]) -> NSDictionary {
        let dict : NSMutableDictionary = [:]
        
        for item in items {
            dict.setValue(item.value, forKey: item.name)
        }
        
        if let value = items.filter({ $0.name == "biometric" }).first?.value {
            if value == "false" {
                persist.set(false, forKey: KEY_BIOMETRIC_STATUS)
            }
            else if value == "true" {
                persist.set(true, forKey: KEY_BIOMETRIC_STATUS)
            }
        }
        if let value = items.filter({ $0.name == "rememberme" }).first?.value {
            if value.lowercased() == "true" || value.lowercased() == "1" {
                persist.set(true, forKey: _mobileRemember)
                if let msisdn = items.filter({ $0.name == "mobile" }).first?.value {
                    KeychainService.savePhone(token: msisdn as NSString)
                }
                else {
                    persist.removeObject(forKey: _mobileRemember)
                    KeychainService.clearPhone()
                }
            }
            else {
                persist.removeObject(forKey: _mobileRemember)
                KeychainService.clearPhone()
            }
        }
        print(dict)
        
        return dict
    }
    
    private func sendValues(url: URL, operation: LCDelegateOperation) {
        
        let queryItems = URLComponents(url: url, resolvingAgainstBaseURL: false)?.queryItems
        
        if let items = queryItems {
            let retDict = parseValues(items)
            
            if retDict.count > 0 {
                switch operation {
                case .success:
                    delegate?.onSuccess(param: retDict)
                    self.dismiss(animated: false, completion: nil)
                case .fail:
                    if (failAlertMode == "MODE_ALERT") {
                        let alert = UIAlertController(title: nil, message: ((retDict["desc"] ?? "Алдаа гарлаа") as! String), preferredStyle: UIAlertController.Style.alert)
                        alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil))
                        self.present(alert, animated: true, completion: nil)
                    } else {
                        showToast((retDict["desc"] ?? "Алдаа гарлаа") as! String)
                    }
                    
                    delegate?.onFail(param: retDict)
                case .goto:
                    delegate?.goto(param: retDict)
                    self.dismiss(animated: false, completion: nil)
                case .redirect:
                    delegate?.redirect(param: retDict)
                default:
                    didFail(desc: "wrong delegate oper", code: 101)
                }
            }
            else {
                didFail(desc: "no data", code: 101)
            }
        }
        else {
            didFail(desc: "no query items", code: 101)
        }
    }
    
    private func didFail(desc: String, code: Int = 101) {
        if (failAlertMode == "MODE_ALERT") {
            let alert = UIAlertController(title: nil, message: desc, preferredStyle: UIAlertController.Style.alert)
            alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil))
            self.present(alert, animated: true, completion: nil)
        } else {
            showToast(desc)
        }
        
        let dict : NSMutableDictionary = [:]
        dict.setValue(desc, forKey: "desc")
        dict.setValue(code, forKey: "code")
        delegate?.onFail(param: dict)
    }
    
    //MARK:- Authentication
    
    private func biometricAuth(url: String, successBlock: @escaping (String) -> Void, failDelegate: ((String, Int) -> Void)?, failLoad: ((String) -> Void)?) {
        let localAuthenticationContext = LAContext()
        
        var authError: NSError?
        let reasonString = "TouchID/FaceID уншуулна уу"
        
        if localAuthenticationContext.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &authError) {
            
            localAuthenticationContext.evaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, localizedReason: reasonString) { success, evaluateError in
                DispatchQueue.main.async() {
                    if success {
                        successBlock(url)
                    } else {
                        guard let error = evaluateError else {
                            failDelegate?("evaluateError is null", 101)
                            return
                        }
                        self.evaluatePolicy(errorCode: error._code, url: url, failDelegate: failDelegate, failLoad: failLoad)
                    }
                }
            }
        } else {
            
            guard let error = authError else {
                failDelegate?("authError is null", 101)
                return
            }
            
            if #available(iOS 11, *){
                switch error.code {
                case LAError.touchIDNotEnrolled.rawValue:
                    print("LAErrorTouchIDNotEnrolled")
                    failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                    failLoad?(url)
                    persist.set(false, forKey: KEY_BIOMETRIC_STATUS)
                case LAError.passcodeNotSet.rawValue:
                    print("PASSCODE is not configured")
                    failDelegate?("Хурууны хээ ашиглахын тулд passcode/pattern/pinpass тохируулах шаардлагатай.", 101)
                    failLoad?(url)
                case LAError.biometryLockout.rawValue:
                    print("Biometery is locked out")
                    failDelegate?("Бүртгэлгүй хурууны хээгээр нэвтрэх үйлдэл хийсэн тул нэвтрэх боломжгүй.", 101)
                    failLoad?(url)
                case LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    print("LAErrorTouchIDNotAvailable")
                    failDelegate?("Хурууны хээ дэмжихгүй төхөөрөмж", 101)
                    failLoad?(url)
                }
            }
            else{
                switch error.code {
                case LAError.touchIDNotEnrolled.rawValue:
                    print("LAErrorTouchIDNotEnrolled")
                    failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                    failLoad?(url)
                case LAError.passcodeNotSet.rawValue:
                    print("PASSCODE is not configured")
                    failDelegate?("Хурууны хээ ашиглахын тулд passcode/pattern/pinpass тохируулах шаардлагатай.", 101)
                    failLoad?(url)
                case LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    print("LAErrorTouchIDNotAvailable")
                    failDelegate?("Хурууны хээ дэмжихгүй төхөөрөмж", 101)
                    failLoad?(url)
                }
            }
        }
    }
    
    func callRecoverLocalAuth(url: String, successBlock: @escaping (String) -> Void, failDelegate: ((String, Int) -> Void)?, failLoad: ((String) -> Void)?){
        if #available(iOS 8, *) {
            let localAuthenticationContext = LAContext()
            
            var authError: NSError?
            let reasonString = "TouchID/FaceID уншуулна уу"
            
            // Biometric baival shalgana
            if localAuthenticationContext.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &authError) {
                localAuthenticationContext.evaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, localizedReason: reasonString) { success, evaluateError in
                    DispatchQueue.main.async() {
                        if success {
                            successBlock(url)
                        } else {
                            guard let error = evaluateError else {
                                failDelegate?("evaluateError is null", 101)
                                return
                            }
                            self.evaluatePolicy(errorCode: error._code, url: url, failDelegate: failDelegate, failLoad: failLoad)
                        }
                    }
                }
            } else {//Passcode baival shalgana
                let reasonString = "Утасны кодоо оруулна уу."
                if #available(iOS 9.0, *) {
                    if localAuthenticationContext.canEvaluatePolicy(.deviceOwnerAuthentication, error: &authError) {
                        localAuthenticationContext.evaluatePolicy(.deviceOwnerAuthentication, localizedReason: reasonString) { success, evaluateError in
                            DispatchQueue.main.async() {
                                if success {
                                    successBlock(url)
                                } else {
                                    guard let error = evaluateError else {
                                        failDelegate?("evaluateError is null", 101)
                                        return
                                    }
                                    self.evaluatePolicy(errorCode: error._code, url: url, failDelegate: failDelegate, failLoad: failLoad)
                                }
                            }
                        }
                    }
                    else{
                        //success
                        successBlock(url)
                    }
                }
                else{
                    //success
                    successBlock(url)
                }
            }
        }
    }
    
    private func evaluatePolicy(errorCode: Int, url: String, failDelegate: ((String, Int) -> Void)?, failLoad: ((String) -> Void)?){
        if #available(iOS 11, *){
            switch(errorCode){
            case LAError.authenticationFailed.rawValue:
                print("Authentication Failed")
                failDelegate?("Authentication Failed", 101)
                failLoad?(url)
            case LAError.userCancel.rawValue:
                print("User pressed Cancel button")
                failDelegate?("User pressed Cancel button", 101)
                failLoad?(url)
            case LAError.userFallback.rawValue:
                print("User pressed \"Enter Password\"")
                failDelegate?("User pressed \"Enter Password\"", 101)
                failLoad?(url)
            case LAError.appCancel.rawValue, LAError.systemCancel.rawValue:
                print("APP CANCELLED")
                failDelegate?("App cancelled", 101)
                failLoad?(url)
            case LAError.touchIDNotEnrolled.rawValue:
                print("LAErrorTouchIDNotEnrolled")
                failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                failLoad?(url)
            case LAError.touchIDNotAvailable.rawValue:
                print("LAErrorTouchIDNotAvailable")
                failDelegate?("Хурууны хээ дэмжихгүй төхөөрөмж", 101)
                failLoad?(url)
            case LAError.passcodeNotSet.rawValue:
                print("PASSCODE is not configured")
                failDelegate?("Хурууны хээ ашиглахын тулд passcode/pattern/pinpass тохируулах шаардлагатай.", 101)
                failLoad?(url)
            case LAError.biometryLockout.rawValue:
                print("Biometery is locked out")
                failDelegate?("Бүртгэлгүй хурууны хээгээр нэвтрэх үйлдэл хийсэн тул нэвтрэх боломжгүй.", 101)
                failLoad?(url)
            default:
                print("Touch ID is not configured")
                failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                failLoad?(url)
            }
        }
        else if #available(iOS 9, *){
            switch(errorCode){
            case LAError.authenticationFailed.rawValue:
                print("Authentication Failed")
                failDelegate?("Authentication Failed", 101)
                failLoad?(url)
            case LAError.userCancel.rawValue:
                print("User pressed Cancel button")
                failDelegate?("User pressed Cancel button", 101)
                failLoad?(url)
            case LAError.userFallback.rawValue:
                print("User pressed \"Enter Password\"")
                failDelegate?("User pressed \"Enter Password\"", 101)
                failLoad?(url)
            case LAError.appCancel.rawValue, LAError.systemCancel.rawValue:
                print("APP CANCELLED")
            case LAError.touchIDNotEnrolled.rawValue:
                print("LAErrorTouchIDNotEnrolled")
                failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                failLoad?(url)
            case LAError.touchIDNotAvailable.rawValue:
                print("LAErrorTouchIDNotAvailable")
                failDelegate?("Хурууны хээ дэмжихгүй төхөөрөмж", 101)
                failLoad?(url)
            case LAError.passcodeNotSet.rawValue:
                print("PASSCODE is not configured")
                failDelegate?("Хурууны хээ ашиглахын тулд passcode/pattern/pinpass тохируулах шаардлагатай.", 101)
                failLoad?(url)
            default:
                print("Touch ID is not configured")
                failDelegate?("Та хурууны хээгээ гар утсан дээрээ тохируулсны дараа хурууны хээгээр нэвтэрнэ үү.", 101)
                failLoad?(url)
            }
        }
        else{
            failDelegate?("Хурууны хээ дэмжихгүй төхөөрөмж", 101)
            failLoad?(url)
        }
    }
    
    // MARK:- Overrides
    override public func viewDidLoad() {
        super.viewDidLoad()
        initControls()
        
        webView.allowsBackForwardNavigationGestures = true
        loadURL()
    }
    
    override public func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    override public func viewWillDisappear(_ animated: Bool) {
        super.viewWillAppear(animated)
        UIApplication.shared.isNetworkActivityIndicatorVisible = false
    }
    
    // MARK:- Layouts
    private func initControls() {
        self.view.addSubview(vwMain)
        self.view.backgroundColor = hexStringToUIColor(hex: "#F9FAFA", alpha: 1.0)
        vwMain.addSubview(webView)
        vwMain.backgroundColor = .white
        
        setConstraints()
    }
    
    private func setConstraints() {
        vwMain.translatesAutoresizingMaskIntoConstraints = false
        vwMain.leadingAnchor.constraint(equalTo: self.view.leadingAnchor).isActive = true
        vwMain.trailingAnchor.constraint(equalTo: self.view.trailingAnchor).isActive = true
        
        if #available(iOS 11.0, *) {
            let guide: UILayoutGuide = self.view.safeAreaLayoutGuide
            vwMain.topAnchor.constraint(equalTo: guide.topAnchor).isActive = true
            vwMain.bottomAnchor.constraint(equalTo: guide.bottomAnchor).isActive = true
        } else {
            vwMain.topAnchor.constraint(equalTo: self.view.topAnchor).isActive = true
            vwMain.bottomAnchor.constraint(equalTo: self.view.bottomAnchor).isActive = true
        }
        
        webView.translatesAutoresizingMaskIntoConstraints = false
        webView.topAnchor.constraint(equalTo: vwMain.topAnchor).isActive = true
        webView.leadingAnchor.constraint(equalTo: vwMain.leadingAnchor).isActive = true
        webView.trailingAnchor.constraint(equalTo: vwMain.trailingAnchor).isActive = true
        webView.bottomAnchor.constraint(equalTo: vwMain.bottomAnchor).isActive = true
    }
    
    private func getZoomDisableScript() -> WKUserScript {
        let source: String = "var meta = document.createElement('meta');" +
            "meta.name = 'viewport';" +
            "meta.content = 'width=device-width, initial-scale=1.0, maximum- scale=1.0, user-scalable=no';" +
            "var head = document.getElementsByTagName('head')[0];" + "head.appendChild(meta);"
        return WKUserScript(source: source, injectionTime: .atDocumentEnd, forMainFrameOnly: true)
    }
    
    // MARK:- Delegates
    public func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        
        if let url = navigationAction.request.url {
            
            let urlString = url.absoluteString.lowercased()
            
            if urlString.contains("/success") {
                sendValues(url: url, operation: .success)

                decisionHandler(.cancel)
                return
            }
            else if urlString.contains("/failed") {
                sendValues(url: url, operation: .fail)

                decisionHandler(.cancel)
                return
            }
            else if urlString.contains("/goto") {
                sendValues(url: url, operation: .goto)

                decisionHandler(.cancel)
                return
            }
            else if urlString.contains("redirect=") {
                sendValues(url: url, operation: .redirect)

                decisionHandler(.allow)
                return
            }
            else if urlString.contains("/fingerprint") {
                biometricAuth(url: "/fingerprint.aspx", successBlock: { (subURL) in
                    if let url = self.externalURL {
                        self.loadURL(url: (url + subURL + "?fpenable=true"))
                    }
                    else {
                        self.didFail(desc: "externalurl is not set", code: 101)
                    }
                }, failDelegate: nil) { (subURL) in
                    if let url = self.externalURL {
                        self.loadURL(url: (url + subURL + "?fpenable=false"))
                    }
                    else {
                        self.didFail(desc: "externalurl is not set", code: 101)
                    }
                }
                decisionHandler(.cancel)
                return
            }
            else if urlString.contains("/sendpinfp") {
                callRecoverLocalAuth(url: "/secquestion.aspx", successBlock: { (subURL) in
                    if let url = self.externalURL {
                        self.loadURL(url: (url + subURL + "?fpenable=true"))
                    }
                    else {
                        self.didFail(desc: "externalurl is not set", code: 101)
                    }
                }, failDelegate: { (desc, code) in
                    self.didFail(desc: desc, code: code)
                }, failLoad: nil)
                decisionHandler(.cancel)
                return
            }
            else if urlString.contains("/loginfp") {
                if let _ = KeychainService.loadPhone() as String?, persist.bool(forKey: _mobileRemember) == true, biometricStatus == true {
                    biometricAuth(url: fullURL, successBlock: { (url) in
                        let biometricURL = url + "&from=biometric"
                        self.loadURL(url: biometricURL)
                    }, failDelegate: { (desc, code) in
                        self.didFail(desc: desc, code: code)
                    }, failLoad: nil)
                }
                decisionHandler(.cancel)
                return
            }
        }

        decisionHandler(.allow)
    }
    
    public func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        if self.navigation == .KEY_NAVIGATION_LOGIN {
            if let _ = KeychainService.loadPhone() as String?, persist.bool(forKey: _mobileRemember) == true, biometricStatus == true {
                biometricAuth(url: fullURL, successBlock: { (url) in
                    let biometricURL = url + "&from=biometric"
                    self.loadURL(url: biometricURL)
                }, failDelegate: { (desc, code) in
                    self.didFail(desc: desc, code: code)
                }, failLoad: nil)
            }
        }
    }
}

extension LoginVC: UIScrollViewDelegate {
    public func scrollViewWillBeginZooming(_ scrollView: UIScrollView, with view: UIView?) {
      scrollView.pinchGestureRecognizer?.isEnabled = false
   }
}

