import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class MmComponent {
  static const MethodChannel _channel = const MethodChannel('mm_component');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static const KEY_BIOMETRIC_STATUS = "KEY_BIOMETRIC_STATUS";
  static const KEY_NAVIGATION_CHANGEPIN = "KEY_NAVIGATION_CHANGEPIN";
  static const KEY_NAVIGATION_PIN = "KEY_NAVIGATION_PIN";
  static const KEY_NAVIGATION_CHANGEMOBILE = "KEY_NAVIGATION_CHANGEMOBILE";
  static const KEY_NAVIGATION_LOGIN = "KEY_NAVIGATION_LOGIN";

  static const MODE_ALERT = "MODE_ALERT";
  static const MODE_TOAST = "MODE_TOAST";

  static Future<void> open({
    @required String operation,
    @required String version,
    @required url,
    @required Function onSuccess,
    @required Function onGoto,
    Function onFail,
    Function onRedirect,
    String failAlertMode = MODE_ALERT,
    bool openNewComponent = true,
  }) async {
    String result = await _channel.invokeMethod('open', {
      "operation": operation,
      "version": version,
      "url": url,
      "openNewComponent": openNewComponent,
      "failAlertMode": failAlertMode,
    });
    dispatchResult(
      result: result,
      operation: operation,
      version: version,
      url: url,
      onSuccess: onSuccess,
      onGoto: onGoto,
      failAlertMode: failAlertMode,
      onFail: onFail,
      onRedirect: onRedirect,
    );
  }

  static void dispatchResult({
    @required String result,
    @required String operation,
    @required String version,
    @required url,
    @required Function onSuccess,
    @required Function onGoto,
    @required String failAlertMode,
    Function onFail,
    Function onRedirect,
  }) {
    Map<String, dynamic> response = jsonDecode(result);

    switch (response["resultCode"]) {
      case "onSuccess":
        onSuccess(response["resultData"].toString());
        break;
      case "onGoto":
        onGoto(response["resultData"].toString());
        break;
      case "onFail":
        open(
          operation: operation,
          version: version,
          url: url,
          onFail: onFail,
          onSuccess: onSuccess,
          onGoto: onGoto,
          onRedirect: onRedirect,
          failAlertMode: failAlertMode,
          openNewComponent: false,
        );
        if (onFail != null) {
          onFail(response["resultData"].toString());
        }
        break;
      case "onRedirect":
        open(
          operation: operation,
          version: version,
          url: url,
          onFail: onFail,
          onSuccess: onSuccess,
          onGoto: onGoto,
          onRedirect: onRedirect,
          failAlertMode: failAlertMode,
          openNewComponent: false,
        );
        if (onRedirect != null) {
          onRedirect(response["resultData"].toString());
        }
        break;
    }
  }
}